@@ -10,10 +10,11 @@ __copyright__ = "Copyright © 2005 Canonical Ltd."
 __author__    = "Scott James Remnant <scott@ubuntu.com>"
 
 
+from bzrlib.revision import Revision
 from bzrlib.tsort import merge_sort
 
 
-class DummyRevision(object):
+class DummyRevision(Revision):
     """Dummy bzr revision.
 
     Sometimes, especially in older bzr branches, a revision is referenced
@@ -21,12 +22,12 @@ class DummyRevision(object):
     When this happens we use an instance of this class instead of the real
     Revision object (which we can't get).
     """
-
     def __init__(self, revid):
-        self.revision_id = revid
-        self.parent_ids = []
+        super(DummyRevision, self).__init__(revid)
         self.committer = None
-        self.message = self.revision_id
+        self.timestamp = None
+        self.timezone = None
+        self.message = revid
 
 
 class RevisionProxy(object):
@@ -63,24 +64,24 @@ class RevisionProxy(object):
 
 class DistanceMethod(object):
 
-    def __init__(self, branch, start):
-        self.branch = branch
-        self.start = start
+    def __init__(self, repository, start_revid):
+        self.repository = repository
+        self.start_revid = start_revid
         self.revisions = {}
         self.children = {}
-        self.children_of_id = {start: set()}
+        self.children_of_id = {start_revid: set()}
         self.parent_ids_of = {}
-        self.colours = { start: 0 }
+        self.colours = { start_revid: 0 }
         self.last_colour = 0
         self.direct_parent_of = {}
         self.graph = {}
 
     def fill_caches(self):
-        graph = self.branch.repository.get_revision_graph_with_ghosts([self.start])
+        graph = self.repository.get_revision_graph_with_ghosts([self.start_revid])
         for revid in graph.ghosts:
             self.cache_revision(DummyRevision(revid))
         for revid, parents in graph.get_ancestors().items():
-            self.cache_revision(RevisionProxy(revid, parents, self.branch.repository))
+            self.cache_revision(RevisionProxy(revid, parents, self.repository))
 
     def cache_revision(self, revision):
         "Set the caches for a newly retrieved revision."""
@@ -235,7 +236,7 @@ class DistanceMethod(object):
                 self.colours[revid] = self.last_colour = self.last_colour + 1
 
 
-def distances(branch, start):
+def distances(repository, start_revid):
     """Sort the revisions.
 
     Traverses the branch revision tree starting at start and produces an
@@ -244,9 +245,9 @@ def distances(branch, start):
 
     Returns a tuple of (revids, revisions, colours, children)
     """
-    distance = DistanceMethod(branch, start)
+    distance = DistanceMethod(repository, start_revid)
     distance.fill_caches()
-    distance.merge_sorted = merge_sort(distance.graph, distance.start)
+    distance.merge_sorted = merge_sort(distance.graph, distance.start_revid)
     children = distance.make_children_map()
     
     for seq, revid, merge_depth, end_of_merge in distance.merge_sorted:
